/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.platform.exceptions;

import java.util.ArrayList;
import java.util.List;

import org.springframework.context.MessageSource;

/**
 * An application exception that transports validation errors.
 * A bean name is always required when creating a BeanValidationException.
 * It can contain a list of BeanValidationException, which in most cases will be
 * a list of {@link PropertyValidationException}.
 */
public class BeanValidationException extends ApplicationException {

    private static final long serialVersionUID = 1L;

    public static final String ERR_VAL = "error.validation";

    private final String beanName;

    private List<BeanValidationException> validationIssues;

    public BeanValidationException(String beanName, String defaultMessage) {
        this(ERR_VAL, beanName, defaultMessage);
    }

    public BeanValidationException(String errorCode, String beanName, String defaultMessage) {
        // Variant prefix the errorCode with the beanName:
        //        super(beanName.substring(0, beanName.lastIndexOf('.')+1) + ERR_VAL, new Object[] {beanName}, defaultMessage);
        this(errorCode, beanName, new Object[] {beanName}, defaultMessage);
    }

    public BeanValidationException(String errorCode, String beanName, Object[] messageArgs, String defaultMessage) {
        super(errorCode, messageArgs, defaultMessage);
        this.beanName = beanName;
    }

    public BeanValidationException addValidationIssue(BeanValidationException valException) {
        if (this.validationIssues == null) {
            this.validationIssues = new ArrayList<BeanValidationException>();
        }
        this.validationIssues.add(valException);
        return this;
    }

    public List<BeanValidationException> getValidationIssues() {
        return this.validationIssues;
    }
    
    public boolean hasValidationIssues() {
        return this.validationIssues != null && !this.validationIssues.isEmpty();
    }

    public String getBeanName() {
        return this.beanName;
    }
}
