package org.openxma.dsl.platform.xma.client;

import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Widget;
import org.openxma.dsl.platform.xma.client.browse.BrowseMode;
import org.openxma.dsl.platform.xma.client.effects.Effect;
import org.openxma.dsl.platform.xma.client.effects.MouseEffect;

import at.spardat.xma.component.ComponentClient;
import at.spardat.xma.mdl.AttachmentExceptionClient;
import at.spardat.xma.mdl.IWModelClient;
import at.spardat.xma.mdl.paging.PagingWMClient;
import at.spardat.xma.mdl.table.ITableWMClient;
import at.spardat.xma.page.DialogPage;
import at.spardat.xma.page.PageClient;

public abstract class DslDialogPage extends DialogPage {

    private String comboLastVal = null;

    public DslDialogPage(ComponentClient component, boolean stateless, int style) {
        super(component, stateless, style);
    }

    public DslDialogPage(ComponentClient component, Shell parentShell, boolean stateless, int style) {
        super(component, parentShell, stateless, style);
    }

    public DslDialogPage(PageClient parent, boolean stateless, int style) {
        super(parent, stateless, style);
    }

    protected Control createBrowseWidget(Control editControl, int style) {
        return BrowseMode.getInstance().createBrowseWidget(editControl, style);
    }

    protected void attachBrowseWidget(IWModelClient wModel, Control browseControl) {
        BrowseMode.getInstance().attachBrowseWidget(wModel, browseControl);
    }

    public void attachBrowseWidgets() {
    }

    public void attachUI() throws AttachmentExceptionClient {
        super.attachUI();
        attachBrowseWidgets();
    }

    public void detachUI() {
        super.detachUI();
        BrowseMode.getInstance().detachBrowseControls(getWModels());
    }

    protected String getAppCode() {
        return "UNDEFINED";
    }

    protected void resetPagingControl(SelectionEvent event, ITableWMClient tableWMClient) {

        if (event != null && event.widget instanceof Combo) {
            Combo combo = (Combo) event.widget;
            String text = combo.getText();
            if (text != null) {
                if (comboLastVal == null || !text.equals(comboLastVal)) {
                    comboLastVal = text;
                    resetPagingControl(tableWMClient);
                }
            }
        } else {
            resetPagingControl(tableWMClient);
        }
    }

    private void resetPagingControl(ITableWMClient tableWMClient) {
        Table table = (Table) tableWMClient.getUIDelegate().getUIControl();
        PagingWMClient pagingControl = (PagingWMClient) table.getData("PAGING_CONTROL");
        pagingControl.setResultSize(Integer.MAX_VALUE);
        pagingControl.setOffset(0);
    }
    
    protected void setEffect(final Widget widget, final Effect effect) {
        if (widget instanceof Control && effect instanceof MouseEffect) {           
            Control control = (Control)widget;
            control.addMouseTrackListener((MouseEffect)effect);
        }
    }
    
    protected void removeEffect(final Widget widget, final Effect effect) {
        if (widget instanceof Control && effect instanceof MouseEffect) {           
            Control control = (Control)widget;
            control.removeMouseTrackListener((MouseEffect)effect);
        }
    }      
}
